
from django.db.models import Manager

from mezzanine.settings import COMMENTS_UNAPPROVED_VISIBLE


class CommentManager(Manager):
    """
    Provides filter for restricting comments that are not approved if 
    COMMENTS_UNAPPROVED_VISIBLE is set to False.
    """

    def visible(self):
        if COMMENTS_UNAPPROVED_VISIBLE:
            return self.all()
        return self.filter(approved=True)




from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.template.loader import select_template

from mezzanine.pages.models import Page


def admin_page_ordering(request):
    """
    Updates the ordering of pages via AJAX from within the admin.
    """
    for i, page in enumerate(request.POST.get("ordering", "").split(",")):
        try:
            Page.objects.filter(id=page.split("_")[-1]).update(ordering=i)
        except ValueError:
            pass
    return HttpResponse("")
admin_page_ordering = staff_member_required(admin_page_ordering)

def page(request, slug, template="pages/page.html"):
    """
    Display content for a page.
    """
    page = get_object_or_404(Page.objects.published(request.user), slug=slug)
    context = {"page": page,}
    templates = ["pages/%s.html" % slug]
    content_model = page.get_content_model()
    if content_model is not None:
        templates.append("pages/%s.html" % content_model)
    templates.append(template)
    t = select_template()
    return HttpResponse(t.render(RequestContext(request, context)))





from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


class OrderWidget(forms.HiddenInput):

    def render(self, *args, **kwargs):
        rendered = super(OrderWidget, self).render(*args, **kwargs)
        arrows = ["<img src='%simg/admin/arrow-%s.gif' />" % 
            (settings.ADMIN_MEDIA_PREFIX, arrow) for arrow in ("up", "down")]
        arrows = "<span class='ordering'>%s</span>" % "".join(arrows)
        return rendered + mark_safe(arrows) 

class OrderableAdminForm(forms.ModelForm):
    _order = forms.CharField(label=_("Order"), widget=OrderWidget, 
        required=False)





from calendar import month_name

from django.contrib.auth.models import User
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.db.models import Count
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.template.loader import select_template

from mezzanine.blog.forms import CommentForm
from mezzanine.blog.models import BlogPost
from mezzanine import settings as blog_settings


use_disqus = bool(blog_settings.COMMENTS_DISQUS_SHORTNAME)


def blog_post_list(request, tag=None, year=None, month=None, username=None, 
    template="blog/blog_post_list.html"):
    """
    Display a list of blog posts.
    """
    blog_posts = BlogPost.objects.published(for_user=request.user).annotate(
        num_comments=Count("comments"))
    if tag is not None:
        blog_posts = blog_posts.filter(keywords__value=tag)
    if year is not None:
        blog_posts = blog_posts.filter(publish_date__year=year) 
        if month is not None:
            blog_posts = blog_posts.filter(publish_date__month=month)
            month = month_name[int(month)]
    user = None
    if username is not None:
        user = get_object_or_404(User, username=username)
        blog_posts = blog_posts.filter(user=user)
    paginator = Paginator(blog_posts, blog_settings.BLOG_POST_PER_PAGE)
    try:
        page_num = int(request.GET.get("page", 1))
    except ValueError:
        page_num = 1
    try:
        blog_posts = paginator.page(page_num)
    except (EmptyPage, InvalidPage):
        blog_posts = paginator.page(paginator.num_pages)
    page_range = blog_posts.paginator.page_range
    max_links = blog_settings.BLOG_POST_MAX_PAGING_LINKS
    if len(page_range) > max_links:
        start = min(blog_posts.paginator.num_pages - max_links, 
            max(0, blog_posts.number - (max_links / 2) - 1))
        page_range = page_range[start:start + max_links]
    context = {"blog_posts": blog_posts, "page_range": page_range, "tag": tag, 
        "year": year, "month": month, "user": user, "use_disqus": use_disqus}
    return render_to_response(template, context, RequestContext(request))

def blog_post_detail(request, slug, template="blog/blog_post_detail.html"):
    """
    Display a blog post.
    """
    # Create two comment forms - one with posted data and errors that will be 
    # matched to the form submitted via comment_id, and an empty one for all 
    # other instances.
    blog_post = get_object_or_404(
        BlogPost.objects.published(for_user=request.user), slug=slug)
    posted_comment_form = CommentForm(request.POST or None)
    unposted_comment_form = CommentForm()
    if request.method == "POST" and posted_comment_form.is_valid():
        comment = posted_comment_form.save(commit=False)
        comment.blog_post = blog_post
        comment.ip_address = request.META.get("HTTP_X_FORWARDED_FOR", 
	        request.META["REMOTE_ADDR"])
        comment.replied_to_id = request.POST.get("replied_to")
        comment.save()
        return HttpResponseRedirect(comment.get_absolute_url())
    context = {"blog_post": blog_post, "use_disqus": use_disqus, 
        "posted_comment_form": posted_comment_form, "unposted_comment_form": 
        unposted_comment_form}
    t = select_template(["blog/%s.html" % slug, template])
    return HttpResponse(t.render(RequestContext(request, context)))





from django.db.models import AutoField
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from mezzanine.settings import CONTENT_MEDIA_URL
from mezzanine.core.forms import OrderableAdminForm


media_url = CONTENT_MEDIA_URL.strip("/")
content_media = lambda files: ["/%s/%s" % (media_url, f) for f in files]

# Build the list of admin JS file for ``Displayable`` models. For Django 1.2 or
# later, include a backport of the collapse js which targets the earlier admin.
displayable_js = ["js/tinymce_setup.js", "js/jquery-1.4.2.min.js",  
    "js/keywords_field.js"]
from django import VERSION
if not (VERSION[0] <= 1 and VERSION[1] <= 1):
    displayable_js.append("js/collapse_backport.js")
displayable_js = content_media(displayable_js)
displayable_js.insert(0, "/media/admin/tinymce/jscripts/tiny_mce/tiny_mce.js")

orderable_js = content_media(["js/jquery-1.4.2.min.js", 
    "js/jquery-ui-1.8.1.custom.min.js", "js/orderable_inline.js"])


class DisplayableAdmin(admin.ModelAdmin):
    """
    Admin model for subclasses of the abstract ``Displayable`` model.
    """

    class Media:
        js = displayable_js
        
    list_display = ("title", "status", "admin_link")
    list_display_links = ("title",)
    list_editable = ("status",)
    list_filter = ("status",)
    search_fields = ("title", "body",)
    date_hierarchy = "publish_date"
    radio_fields = {"status": admin.HORIZONTAL}
    fieldsets = (
        (None, {"fields": ("title", ("status", "publish_date"), "content")}),
        (_("Meta data"), {"fields": ("description", "keywords"), 
            "classes": ("collapse-closed",)},),
    )

class OrderableAdmin(admin.ModelAdmin):
    """
    Admin model that handles inlines for models that subclass the abstract 
    ``Orderable`` model.
    """

    class Media:
        css = {"all": content_media(["css/orderable_inline.css"])}
        js = orderable_js
    
    def __init__(self, *args, **kwargs):
        """
        Set the form for each of the inlines, the extras that will be hidden 
        and also ensure the ``_order`` field is last.
        """
        for inline in self.inlines:
            inline.form = OrderableAdminForm
            inline.extra = 20
            fields = inline.fields
            if not fields:
                fields = inline.model._meta.fields
                exclude = inline.exclude or []
                fields = [f.name for f in fields if f.editable and 
                    f.name not in exclude and not isinstance(f, AutoField)]
            if "_order" in fields:
                del fields[fields.index("_order")]
                fields.append("_order")
            inline.fields = fields
        super(OrderableAdmin, self).__init__(*args, **kwargs)






from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponse

from mezzanine.core.models import Keyword


def admin_keywords_submit(request):
    """
    Adds any new given keywords from the custom keywords field in the admin and 
    returns their IDs for use when saving a model with a keywords field.
    """
    ids = []
    for value in request.POST.get("text_keywords", "").split(","):
        value = "".join([c for c in value if c.isalnum() or c == "-"]).lower()
        if value:
            keyword, created = Keyword.objects.get_or_create(value=value)
            ids.append(str(keyword.id))
    return HttpResponse(",".join(set(ids)))
admin_keywords_submit = staff_member_required(admin_keywords_submit)





from django.contrib import admin
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404

from mezzanine.pages.models import Page
from mezzanine.core.admin import DisplayableAdmin


class PageAdmin(DisplayableAdmin):
    """
    Admin class for the ``Page`` model and all subclasses of ``Page``. Handles 
    redirections between admin interfaces for the ``Page`` model and its 
    subclasses.
    """

    def change_view(self, request, object_id, extra_context=None):
        """
        For the ``Page`` model, check ``page.get_content_model()`` for a 
        subclass and redirect to its admin change view.
        """
        if self.model is Page:
            page = get_object_or_404(Page, pk=object_id)
            content_model = page.get_content_model()
            if content_model is not None:
                app = content_model.__class__._meta.app_label
                name = content_model.__class__.__name__.lower()
                change_url = reverse("admin:%s_%s_change" % (app, name), 
                    args=(content_model.id,))
                return HttpResponseRedirect(change_url)
        return super(PageAdmin, self).change_view(request, object_id, 
            extra_context=None)

    def changelist_view(self, request, extra_context=None):

        """
        Redirect to the ``Page`` changelist view for ``Page`` subclasses.
        """
        if self.model is not Page:
            return HttpResponseRedirect(reverse("admin:pages_page_changelist"))
        return super(PageAdmin, self).changelist_view(request, extra_context)

    def save_model(self, request, obj, form, change):
        """
        Set the ID of the parent page if passed in via querystring.
        """
        obj.parent_id = request.GET.get("parent")
        super(PageAdmin, self).save_model(request, obj, form, change)
    
    def maintain_parent(self, request, response):
        """
        Maintain the parent ID in the querystring for response_add and 
        response_change.
        """
        location = response._headers["location"][1]
        parent = request.GET.get("parent")
        if parent is not None and "?" not in location:
            location += "?parent=%s" % parent
        return HttpResponseRedirect(location) 

    def response_add(self, request, obj):
        """
        Maintain the parent ID in the querystring.
        """
        response = super(PageAdmin, self).response_add(request, obj)
        return self.maintain_parent(request, response)

    def response_change(self, request, obj):
        """
        Maintain the parent ID in the querystring.
        """
        response = super(PageAdmin, self).response_change(request, obj)
        return self.maintain_parent(request, response)

admin.site.register(Page, PageAdmin)




from django.template import TemplateDoesNotExist
from django.template.loader import get_template

from mezzanine.settings import MOBILE_USER_AGENTS


class MobileTemplate(object):
    """
    If a mobile user agent is detected, inspect the default args for the view 
    func, and if a template name is found assume it is the template arg and 
    attempt to load a mobile template based on the original template name.
    """

    def process_view(self, request, view_func, view_args, view_kwargs):
        user_agent = request.META.get("HTTP_USER_AGENT", "")
        if [check for check in MOBILE_USER_AGENTS if check in user_agent]:
            template = view_kwargs.get("template")
            if template is None:
                for default in view_func.func_defaults:
                    if str(default).endswith(".html"):
                        template = default
            if template is not None:
                template = template.rsplit(".html", 1)[0] + ".mobile.html"
                try:
                    get_template(template)
                except TemplateDoesNotExist:
                    pass
                else:
                    view_kwargs["template"] = template
                    return view_func(request, *view_args, **view_kwargs)
        return None




from collections import defaultdict
from datetime import datetime, timedelta
from time import timezone
from urllib import urlopen, urlencode

from django import template
from django.contrib.auth.models import User
from django.db.models import Count
from django.utils.simplejson import loads

from mezzanine import settings as blog_settings
from mezzanine.blog.forms import BlogPostForm
from mezzanine.blog.models import Comment, BlogPost
from mezzanine.core.models import Keyword
from mezzanine.core.templatetags.mezzanine_tags import register_as_tag


register = template.Library()


@register.simple_tag
def gravatar_url(email_hash, size=32):
    """
    Return the full URL for a Gravatar given an email hash.
    """
    return "http://www.gravatar.com/avatar/%s?s=%s" % (email_hash, size)

thread_template = "blog/includes/comments_thread.html"
@register.inclusion_tag(thread_template, takes_context=True)
def blog_comments_for(context, parent):
    """
    Return a list of child comments for the given parent, storing all 
    comments in a dict in the context when first called using parents as keys 
    for retrieval on subsequent recursive calls from the comments template.
    """
    if "blog_comments" not in context:
        comments = defaultdict(list)
        for comment in parent.comments.visible():
            comments[comment.replied_to_id].append(comment)
        context["blog_comments"] = comments
        parent = None
    else:
        parent = parent.id
    try:
        replied_to = int(context["request"].POST["replied_to"])
    except KeyError:
        replied_to = 0
    context.update({
        "comments_thread": context["blog_comments"].get(parent, []),
        "no_comments": parent is None and not comments,
        "comments_default_approved": blog_settings.COMMENTS_DEFAULT_APPROVED,
        "replied_to": replied_to,
    })
    return context

@register_as_tag(register)
def blog_months(*args):
    """
    Put a list of dates for blog posts into the template context.
    """
    return BlogPost.objects.published().dates("publish_date", "month", 
        order="DESC")

@register_as_tag(register)
def blog_tags(*args):
    """
    Put a list of tags (keywords) for blog posts into the template context.
    """
    tags = Keyword.objects.filter(blogpost__isnull=False).annotate(
        post_count=Count("blogpost"))
    if not tags:
        return []
    counts = [tag.post_count for tag in tags]
    min_count, max_count = min(counts), max(counts)
    sizes = blog_settings.TAG_CLOUD_SIZES
    step = (max_count - min_count) / (sizes - 1)
    if step == 0:
        steps = [sizes / 2]
    else:
        steps = range(min_count, max_count, step)[:sizes]
    for tag in tags:
        c = tag.post_count
        diff = min([(abs(s - c), (s - c)) for s in steps])[1]
        tag.weight = steps.index(c + diff) + 1
    return tags

@register_as_tag(register)
def blog_authors(*args):
    """
    Put a list of authors (users) for blog posts into the template context.
    """
    blog_posts = BlogPost.objects.published()
    return User.objects.filter(blog_posts__in=blog_posts).distinct()

@register_as_tag(register)
def quick_blog_form(*args):
    """
    Puts the quick blog form into the admin dashboard.
    """
    return BlogPostForm()

DISQUS_FORUM_ID = None
recent_comments_template = "admin/includes/recent_comments.html"
@register.inclusion_tag(recent_comments_template, takes_context=True)
def recent_comments(context):
    """
    If the COMMENTS_DISQUS_SHORTNAME and COMMENTS_DISQUS_KEY settings have been 
    set, pull the latest comments in from the Disqus API, using the global 
    DISQUS_FORUM_ID so that this lookup occurs only once, transforming each 
    comment into a dict that looks like the built-in comments model. If these 
    are not set then use the built-in comments model.
    """

    global DISQUS_FORUM_ID
    disqus_key = blog_settings.COMMENTS_DISQUS_KEY
    disqus_shortname = blog_settings.COMMENTS_DISQUS_SHORTNAME
    latest = blog_settings.COMMENTS_NUM_LATEST
    context["comments"] = []
    post_from_comment = lambda comment: int(comment["thread"]["identifier"][0])

    def disqus_api(method, **args):
        """
        Make a call to the Disqus API, parsing the JSON data into Python.
        """
        args.update({"user_api_key": disqus_key, "api_version": "1.1"})
        url = "http://disqus.com/api/%s/?%s" % (method, urlencode(args))
        response = loads(urlopen(url).read())
        return response["message"] if response["succeeded"] else []
    
    if disqus_shortname and disqus_key:
        if DISQUS_FORUM_ID is None:
            forums = disqus_api("get_forum_list")
            for forum in forums:
                if forum["shortname"] == disqus_shortname:
                    DISQUS_FORUM_ID = forum["id"]
        if DISQUS_FORUM_ID is not None:
            comments = disqus_api("get_forum_posts", forum_id=DISQUS_FORUM_ID, 
                limit=latest, exclude="spam,killed")
            posts = BlogPost.objects.in_bulk(map(post_from_comment, comments))
            for comment in comments:
                try:
                    blog_post = posts[post_from_comment(comment)]
                except KeyError:
                    blog_post = None
                context["comments"].append({
                    "name": comment["author"]["display_name"],
                    "email": comment["author"]["email"],
                    "email_hash": comment["author"]["email_hash"],
                    "body": comment["message"],
                    "time_created": datetime.strptime(comment["created_at"], 
                        "%Y-%m-%dT%H:%M") - timedelta(seconds=timezone),
                    "post": blog_post,
                })
    else:
        context["comments"] = Comment.objects.all().select_related(
            ).order_by("-id")[:latest]
    return context




#!/usr/bin/env python

"""
Converts the last backward-compatible grappelli branch into a newly named 
package ``grappelli_safe``.
"""

import os

branch_url = "http://django-grappelli.googlecode.com/svn/branches/grappelli_2"
package_name_from = branch_url.split("/")[-1]
package_name_to = "grappelli_safe"

if not os.path.exists(package_name_from):
    print "Checking out branch..."
    os.system("svn co %s" % branch_url)

for (dirpath, dirnames, filenames) in os.walk(package_name_from, False):
    for name in dirnames:
        path = os.path.join(dirpath, name)
        # Delete SVN directories.
        if ".svn" in path:
            print "Deleting svn %s" % path
            os.rmdir(path)
    for name in filenames:
        path = os.path.join(dirpath, name)
        # Delete SVN files.
        if ".svn" in path:
            print "Deleting svn %s" % path
            os.remove(path)
        elif path.endswith(".py"):
            update = False
            with open(path, "r") as f:
                data = f.read()
            if name == "admin.py":
                # Comment out calls to admin.site.register.
                update = True
                data = data.replace("\nadmin.site.register", 
                    "\n#admin.site.register")
            # Replace these instances of the package name with the new name.
            for replace_str in ("grappelli.", "app_label = \"grappelli\""):
                if replace_str in data:
                    update = True
                    data = data.replace(replace_str, 
                        replace_str.replace("grappelli", package_name_to))
            if update:
                print "Rewriting %s" % path
                with open(path, "w") as f:
                    f.write(data)

# Move the package to the Mezzanine directory using the grappelli_safe name.
script_path = os.path.dirname(os.path.abspath(__file__))
os.renames(package_name_from, os.path.join(script_path, "..", "..", 
    package_name_to))




from collections import defaultdict

from django import template

from mezzanine.core.templatetags.mezzanine_tags import register_as_tag
from mezzanine.twitter.models import Tweet


register = template.Library()


def tweets_for(type, args, per_user=None):
    """
    Retrieve tweets for a user, list or search term. The per user arg limits 
    the number of tweets per user, for example to allow a fair spread of 
    tweets per user for a list.
    """
    lookup = {}
    lookup[type] = args[0].strip("\"'")
    tweets = Tweet.objects.get_for(**lookup)
    if per_user is not None:
        _tweets = defaultdict(list)
        for tweet in tweets:
            if len(_tweets[tweet.user_name]) < per_user:
                _tweets[tweet.user_name].append(tweet)
        tweets = sum(_tweets.values(), [])
        tweets.sort(key=lambda t: t.created_at, reverse=True)
    if len(args) > 1 and args[-1].isdigit():
        tweets = tweets[:int(args[-1])]
    return tweets

@register_as_tag(register)
def tweets_for_user(*args):
    return tweets_for("user_name", args)

@register_as_tag(register)
def tweets_for_list(*args):
    return tweets_for("list_name", args, per_user=1)

@register_as_tag(register)
def tweets_for_search(*args):
    return tweets_for("search_term", args)





from django.shortcuts import get_object_or_404

#from datetime import datetime
#from os.path import join

#from django.core.files.storage import default_storage
#from django.core.urlresolvers import reverse
#from django.core.mail import EmailMessage
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect

from mezzanine.forms.models import Form
#from mezzanine.forms.utils import get_built_form
#from mezzanine.forms.forms import get_built_form_form
#from mezzanine.forms.settings import UPLOAD_TO, EMAIL_TO


def built_form_detail(request, slug, template="forms/built_form_detail.html"):
    """
    display a built form and handle submission
    """
    
    built_form = get_object_or_404(Form.objects.published(request.user), slug=slug)

    built_form = Form.objects.published(for_user=request.user)
    #built_form = get_built_form(request, slug)

    if request.method == "POST":
        
        built_form_form = get_built_form_form(built_form)(
            request.POST, request.FILES)
        if built_form_form.is_valid():
            
            # add the form and date to the submmission data
            built_form_form.cleaned_data.update({"submission_date": 
                datetime.now(), "built_form_id": built_form.id})
                    
            # create the submission and save any files
            attachments = []
            submission = BuiltFormSubmission(**built_form_form.cleaned_data)
            for name, file in built_form_form.files.items():
                filename = default_storage.save("%s/%s" % (UPLOAD_TO, 
                    file.name), file)
                setattr(submission, name, filename)
                attachments.append(filename)
            submission.save()

            # send email if specified
            fields = []
            if built_form.send_email and EMAIL_TO:
                for field in submission._meta.fields:
                    value = getattr(submission, field.name)
                    if value:
                        fields.append("%s: %s" % (field.verbose_name, value))
                msg = EmailMessage(submission, "\n".join(fields), 
                    submission.email, [EMAIL_TO])
                for attachment in attachments:
                    msg.attach_file(join(default_storage.location, attachment))
                msg.send(fail_silently=False)

            return HttpResponseRedirect(reverse("built_form_sent", 
                kwargs={"slug": built_form.slug}))
                    
    else:
        
        # pre-populate relevant fields if user is logged in
        initial = {}
        if request.user.is_authenticated():
            initial = {"first_name": request.user.first_name, "last_name": 
                request.user.last_name, "email": request.user.email}
        #built_form_form = get_built_form_form(built_form)(initial=initial)
        
    return render_to_response(template, {"built_form": built_form, 
        "built_form_form": None}, RequestContext(request))


def built_form_sent(request, slug, template="forms/built_form_sent.html"):
    """
    show the response message
    """

    return render_to_response(template, {"built_form": 
        get_built_form(request, slug)}, RequestContext(request))




from django.db.models import Manager


class TweetManager(Manager):

    def get_for(self, user_name=None, list_name=None, search_term=None):
        """
        Create a query and run it for the given arg if it doesn't exist, and 
        return the tweets for the query.
        """
        if user_name is not None:
            type, value = "user", user_name
        elif list_name is not None:
            type, value = "list", list_name
        elif search_term is not None:
            type, value = "search", search_term
        else:
            return
        from mezzanine.twitter.models import Query
        query, created = Query.objects.get_or_create(type=type, value=value)
        if created:
            query.run()
        elif not query.interested:
            query.interested = True
            query.save()
        return query.tweets.all()





from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import connection
from django.template import Context, Template
from django.test import TestCase

from mezzanine.blog.models import BlogPost, Comment
from mezzanine.pages.models import Page
from mezzanine.settings import BLOG_SLUG, CONTENT_STATUS_DRAFT, \
    CONTENT_STATUS_PUBLISHED, MOBILE_USER_AGENTS


class Tests(TestCase):
    """
    Mezzanine tests.
    """
    
    fixtures = ["initial_data.json"]
    
    def setUp(self):
        """
        Create an admin user.
        """
        self._username = "test"
        self._password = "test"
        args = self._username, "example@example.com", self._password
        self._user = User.objects.create_superuser(*args)

    def test_draft_page(self):
        """
        Test a draft page as only being viewable by a staff member.
        """
        self.client.logout()
        draft = Page.objects.create(title="Draft", status=CONTENT_STATUS_DRAFT)
        response = self.client.get(draft.get_absolute_url())
        self.assertEqual(response.status_code, 404)
        self.client.login(username=self._username, password=self._password)
        response = self.client.get(draft.get_absolute_url())
        self.assertEqual(response.status_code, 200)

    def test_overriden_page(self):
        """
        Test that a page with a slug matching a non-page urlpattern return 
        True for its overriden property. The blog page from the fixtures 
        should classify as this case.
        """
        self.assertTrue(Page.objects.get(slug=BLOG_SLUG).overridden())

    def test_mobile_middleware(self):
        """
        Test that an alternate template is rendered when a mobile device is 
        used.
        """
        default = self.client.get(reverse("home")).template[0].name
        mobile = self.client.get(reverse("home"), 
            HTTP_USER_AGENT=MOBILE_USER_AGENTS[0]).template[0].name
        self.assertNotEqual(default, mobile)
        self.assertEqual(default, mobile.replace(".mobile", "", 1))

    def test_blog_views(self):
        """
        Basic status code test for blog views.
        """
        response = self.client.get(reverse("blog_post_list"))
        self.assertEqual(response.status_code, 200)
        blog_post = BlogPost.objects.create(title="Post", user=self._user,
            status=CONTENT_STATUS_PUBLISHED)
        response = self.client.get(blog_post.get_absolute_url())
        self.assertEqual(response.status_code, 200)

    def queries_used_for_template(self, template, **context):
        """
        Return the number of queries used when rendering a template string.
        """
        settings.DEBUG = True
        connection.queries = []
        t = Template(template) 
        t.render(Context(context))
        settings.DEBUG = False
        return len(connection.queries)
        
    def create_recursive_objects(self, model, parent_field, **kwargs):
        """
        Create multiple levels of recursive objects.
        """
        per_level = range(3)
        for _ in per_level:
            kwargs[parent_field] = None
            level1 = model.objects.create(**kwargs)
            for _ in per_level:
                kwargs[parent_field] = level1
                level2 = model.objects.create(**kwargs)
                for _ in per_level:
                    kwargs[parent_field] = level2
                    model.objects.create(**kwargs)

    def test_comments(self):
        """
        Test that rendering the blog comments executes the same number of 
        queries regardless of the number of nested replies.
        """
        blog_post = BlogPost.objects.create(title="Post", user=self._user)
        template = "{% load blog_tags %}{% blog_comments_for blog_post %}"
        before = self.queries_used_for_template(template, blog_post=blog_post)
        self.create_recursive_objects(Comment, "replied_to", name="Comment", 
            blog_post=blog_post)
        after = self.queries_used_for_template(template, blog_post=blog_post)
        self.assertEquals(before, after)

    def test_page_menu(self):
        """
        Test that rendering the page menu executes the same number of queries 
        regardless of the number of pages or levels of children.
        """
        template = "{% load pages_tags %}{% page_menu %}"
        before = self.queries_used_for_template(template)
        self.create_recursive_objects(Page, "parent", title="Page", 
            status=CONTENT_STATUS_PUBLISHED)
        after = self.queries_used_for_template(template)
        self.assertEquals(before, after)





from django import forms
from django.db import models
from django.utils.translation import ugettext_lazy as _

from mezzanine.core.models import Orderable
from mezzanine.pages.models import Page


FIELD_MAPPING = (
    (1, "Check-box", "BooleanField"),
    (2, "Text", "CharField"),
    (3, "Email", "EmailField"),
    (4, "Date", "DateField"),
    (5, "Date/time", "DateTimeField"),
    (6, "Drop-down", "ChoiceField"),
    (7, "Multi-select", "MultipleChoiceField"),
)

FIELD_CHOICES = [f[:2] for f in FIELD_MAPPING]

class Form(Page):
    """
    A user-built form
    """

    response = models.TextField(_("Response"), max_length=2000)
    send_email = models.BooleanField(_("Send email"), default=True)

    class Meta:
        verbose_name = _("Form")
        verbose_name_plural = _("Forms")
        
    @models.permalink
    def get_absolute_url(self):
        return ("form_detail", (), {"slug": self.slug})

class Field(Orderable):
    
    form = models.ForeignKey("Form")
    label = models.CharField(_("Label"), max_length=20)
    type = models.IntegerField(_("Type"), choices=FIELD_CHOICES)
    mandatory = models.BooleanField(_("Mandatory"))
    choices = models.CharField(_("Choices"), max_length=1000, blank=True)

    class Meta:
        verbose_name = _("Field")
        verbose_name_plural = _("Fields")
        order_with_respect_to = "form"
    
    def __unicode__(self):
        return self.label
    
#class FormSubmission(models.Model):
#    """
#    a website submission to a user-built form
#    """
#    
#    class Meta:
#        verbose_name = "Form entry"
#        verbose_name_plural = "Form entries"
#    
#    built_form = models.ForeignKey(BuiltForm)
#    submission_date = models.DateTimeField()

#    # mandatory form fields for every form
#    first_name = models.CharField(max_length=50)
#    last_name = models.CharField(max_length=50)
#    email = models.EmailField(max_length=50)
#    
#    # extra fields for a form that can be user-specified as mandatory or option
#    # identified by blank=True
#    dob = models.DateField(blank=True, null=True)
#    phone = models.CharField(blank=True, max_length=20)
#    message = models.TextField(blank=True, max_length=2000)
#    image = models.ImageField(blank=True, upload_to=UPLOAD_TO, max_length=50)
#    
#    def __unicode__(self):
#        return "%s Submission: %s %s" % (self.built_form, self.first_name, 
#            self.last_name)

#class FieldSubmission(models.Model):

#    




from django import forms

from mezzanine.blog.models import Comment, BlogPost


class CommentForm(forms.ModelForm):
    """
    Model form for comments against a blog post.
    """

    class Meta:
        model = Comment
        fields = ("name", "email", "website", "body",)

class BlogPostForm(forms.ModelForm):
    """
    Model form for blog post that provides the quick blog panel in the admin 
    dashboard. 
    """

    class Meta:
        model = BlogPost
        fields = ("title", "content", "status")

    def __init__(self):
        super(BlogPostForm, self).__init__()
        self.fields["status"].widget = forms.HiddenInput()





from datetime import datetime
from hashlib import md5

from django.db import models
from django.template.defaultfilters import truncatewords_html
from django.utils.translation import ugettext, ugettext_lazy as _

from mezzanine.core.models import Displayable
from mezzanine.blog.managers import CommentManager
from mezzanine.settings import COMMENTS_DEFAULT_APPROVED


class BlogPost(Displayable):
    """
    A blog post.
    """

    user = models.ForeignKey("auth.User", verbose_name=_("Author"), 
        related_name="blog_posts")

    class Meta:
        verbose_name = "Blog post"
        verbose_name_plural = "Blog posts"
        ordering = ("-publish_date",)

    @models.permalink
    def get_absolute_url(self):
        return ("blog_post_detail", (), {"slug": self.slug})

class Comment(models.Model):
    """
    A comment against a blog post.
    """

    name = models.CharField(_("Name"), max_length=100, help_text=_("required"))
    email = models.EmailField(_("Email"), 
        help_text=_("required (not published)"))
    email_hash = models.CharField(_("Email hash"), max_length=100, blank=True)
    body = models.TextField(_("Comment"))
    website = models.URLField(_("Website"), blank=True, help_text=_("optional"))
    blog_post = models.ForeignKey("BlogPost", related_name="comments")
    approved = models.BooleanField(_("Approved"), 
        default=COMMENTS_DEFAULT_APPROVED)
    ip_address = models.IPAddressField(_("IP address"), blank=True, null=True)
    time_created = models.DateTimeField(_("Created at"), default=datetime.now)
    replied_to = models.ForeignKey("self", blank=True, null=True, 
        related_name="comments")

    objects = CommentManager()

    class Meta:
        verbose_name = _("Comment")
        verbose_name_plural = _("Comments")
        ordering = ("time_created",)

    def __unicode__(self):
        return self.body
        
    def get_absolute_url(self):
        return "%s#comment-%s" % (self.blog_post.get_absolute_url(), self.id)

    def save(self, *args, **kwargs):
        if not self.email_hash:
            self.email_hash = md5(self.email).hexdigest()
        super(Comment, self).save(*args, **kwargs)

    ################################
    # Admin listing column methods #
    ################################

    def intro(self):
        return truncatewords_html(self.body, 20)
    intro.short_description = _("Comment")

    def avatar_link(self):
        from mezzanine.blog.templatetags.blog_tags import gravatar_url
        return "<a href='mailto:%s'><img style='vertical-align:middle;" \
            "margin-right:3px;' src='%s' />%s</a>" % (self.email, 
            gravatar_url(self.email_hash), self.name)
    avatar_link.allow_tags = True
    avatar_link.short_description = ""

    def admin_link(self):
        return "<a href='%s'>%s</a>" % (self.get_absolute_url(), 
            ugettext("View on site"))
    admin_link.allow_tags = True
    admin_link.short_description = ""





from urlparse import urlsplit

from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin
from django.contrib.admin.sites import NotRegistered
from django.views.generic.simple import direct_to_template

from mezzanine.settings import ADMIN_REMOVAL, CONTENT_MEDIA_PATH, \
    CONTENT_MEDIA_URL, PACKAGE_NAME_FILEBROWSER, PACKAGE_NAME_GRAPPELLI

urlpatterns = patterns("",
    url("^admin_keywords_submit/$", 
        "mezzanine.core.views.admin_keywords_submit", 
        name="admin_keywords_submit"),
    ("^%s/(?P<path>.*)$" % CONTENT_MEDIA_URL.strip("/"), 
        "django.views.static.serve", {'document_root': CONTENT_MEDIA_PATH}),
)

# Remove unwanted models from the admin that are installed by default with 
# third-party apps.
for model in ADMIN_REMOVAL:
    try:
        model = tuple(model.rsplit(".", 1))
        exec "from %s import %s" % model
    except ImportError:
        pass
    else:
        try:
            admin.site.unregister(eval(model[1]))
        except NotRegistered:
            pass

# Pairs of optional app names and their urlpatterns.
OPTIONAL_APP_PATTERNS = (
    (PACKAGE_NAME_FILEBROWSER, patterns("",
        ("^admin/filebrowser/", include("%s.urls" % PACKAGE_NAME_FILEBROWSER)),
        ("^%s/(?P<path>.*)$" % getattr(settings, 
            "FILEBROWSER_URL_FILEBROWSER_MEDIA", "").strip("/"), 
            "django.views.static.serve", {'document_root': 
            getattr(settings, "FILEBROWSER_PATH_FILEBROWSER_MEDIA", "")}),
    )),
    (PACKAGE_NAME_GRAPPELLI, patterns("",
        ("^grappelli/", include("%s.urls" % PACKAGE_NAME_GRAPPELLI)),
        ("^%s/admin/(?P<path>.*)$" % urlsplit(settings.ADMIN_MEDIA_PREFIX
            ).path.strip("/").split("/")[0], "django.views.static.serve", 
            {'document_root': getattr(settings, "GRAPPELLI_MEDIA_PATH", "")}),
    )),
)

# Add patterns for optionally installed apps.
for (app, app_patterns) in OPTIONAL_APP_PATTERNS:
    if app in settings.INSTALLED_APPS:
        urlpatterns += app_patterns

# Homepage.
urlpatterns += patterns("",
    url("^$", direct_to_template, {"template": "index.html"}, name="home"),
)




# Main Django settings.
DEBUG = False
DEV_SERVER = False
MANAGERS = ADMINS = ()
TIME_ZONE = "Australia/Melbourne"
SESSION_EXPIRE_AT_BROWSER_CLOSE = True
LANGUAGE_CODE = "en"
SITE_ID = 1
USE_I18N = False
SECRET_KEY = "5tve)^cpj9gsdfg54445364TW#$%u#@$%3sdfqwtjiwxzzt%g6p"
INTERNAL_IPS = ("127.0.0.1",)
TEMPLATE_LOADERS = (
    "django.template.loaders.filesystem.load_template_source",
    "django.template.loaders.app_directories.load_template_source",
)

# Database.
DATABASE_ENGINE = ""
DATABASE_NAME = ""
DATABASE_USER = ""
DATABASE_PASSWORD = ""
DATABASE_HOST = ""
DATABASE_PORT = ""

# Paths.
import os
project_path = os.path.dirname(os.path.abspath(__file__))
project_dir = project_path.split(os.sep)[-1]
MEDIA_URL = "/site_media/"
MEDIA_ROOT = os.path.join(project_path, MEDIA_URL.strip("/"))
TEMPLATE_DIRS = (os.path.join(project_path, "templates"),)
ADMIN_MEDIA_PREFIX = "/media/"
ROOT_URLCONF = "%s.urls" % project_dir
CACHE_MIDDLEWARE_KEY_PREFIX = project_dir

# Apps.
INSTALLED_APPS = (
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.redirects",
    "mezzanine.core",
    "mezzanine.blog",
    "mezzanine.pages",
    "mezzanine.twitter",
    "mezzanine.forms",
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.core.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.request",
)

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.middleware.cache.UpdateCacheMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.cache.FetchFromCacheMiddleware",
    "mezzanine.core.middleware.MobileTemplate",
)

# Optional apps.
from mezzanine.settings import PACKAGE_NAME_FILEBROWSER, PACKAGE_NAME_GRAPPELLI

OPTIONAL_APPS = (
    "debug_toolbar",
    "south",
    "django_extensions", 
    PACKAGE_NAME_FILEBROWSER,
    PACKAGE_NAME_GRAPPELLI,
)

import sys
if not (len(sys.argv) > 1 and sys.argv[1] == "test"):
    for app in OPTIONAL_APPS:
        try:
            __import__(app)
        except ImportError:
            pass
        else:
            INSTALLED_APPS += (app,)
INSTALLED_APPS = sorted(list(INSTALLED_APPS), reverse=True)

# Optional app settings.
if "debug_toolbar" in INSTALLED_APPS:
    DEBUG_TOOLBAR_CONFIG = {"INTERCEPT_REDIRECTS": False}
    MIDDLEWARE_CLASSES += ("debug_toolbar.middleware.DebugToolbarMiddleware",)
if PACKAGE_NAME_GRAPPELLI in INSTALLED_APPS:
    GRAPPELLI_ADMIN_HEADLINE = "Mezzanine"
    GRAPPELLI_ADMIN_TITLE = "Mezzanine"
    GRAPPELLI_MEDIA_PATH = os.path.join(os.path.dirname(
        __import__(PACKAGE_NAME_GRAPPELLI).__file__), "media")
if PACKAGE_NAME_FILEBROWSER in INSTALLED_APPS:
    FILEBROWSER_URL_FILEBROWSER_MEDIA = "/filebrowser/media/"
    FILEBROWSER_PATH_FILEBROWSER_MEDIA = os.path.join(os.path.dirname(
        __import__(PACKAGE_NAME_FILEBROWSER).__file__), "media", "filebrowser")

# Caching.
CACHE_BACKEND = ""
CACHE_TIMEOUT = CACHE_MIDDLEWARE_SECONDS = 0
try:
    import cmemcache
except ImportError:
    try:
        import memcache
    except ImportError:
        CACHE_BACKEND = "locmem:///"
if not CACHE_BACKEND:
    CACHE_TIMEOUT = CACHE_MIDDLEWARE_SECONDS = 180
    CACHE_BACKEND = "memcached://127.0.0.1:11211/?timeout=%s" % CACHE_MIDDLEWARE_SECONDS
    CACHE_MIDDLEWARE_ANONYMOUS_ONLY = True

# Local settings.
try:
    from local_settings import *
except ImportError:
    pass

TEMPLATE_DEBUG = DEBUG
if DEV_SERVER and PACKAGE_NAME_GRAPPELLI in INSTALLED_APPS:
    ADMIN_MEDIA_PREFIX = "http://127.0.0.1:8000/media/admin/"
if DATABASE_ENGINE == "sqlite3":
    DATABASE_NAME = os.path.join(project_path, DATABASE_NAME)





from datetime import datetime

from django.db.models import Manager

from mezzanine.settings import CONTENT_STATUS_PUBLISHED


class PublishedManager(Manager):
    """
    Provides filter for restricting items returned by status and publish date 
    when the given user is not a staff member.
    """
    
    def published(self, for_user=None):
        if for_user is not None and for_user.is_staff:
            return self.all()
        return self.filter(status=CONTENT_STATUS_PUBLISHED, 
            publish_date__lte=datetime.now())




from django.contrib import admin

from mezzanine.blog.models import BlogPost, Comment
from mezzanine.core.admin import DisplayableAdmin
from mezzanine.settings import COMMENTS_DISQUS_SHORTNAME


class BlogPostAdmin(DisplayableAdmin):

    list_display = ("title", "user", "status", "admin_link")

    def save_model(self, request, obj, form, change):
        """
        Set the author as the logged in user.
        """
        obj.user = request.user
        super(BlogPostAdmin, self).save_model(request, obj, form, change)

class CommentAdmin(admin.ModelAdmin):

    list_display = ("avatar_link", "intro", "time_created", "approved", 
        "blog_post", "admin_link")
    list_display_links = ("intro", "time_created")
    list_editable = ("approved",)
    list_filter = ("blog_post", "approved", "name")
    search_fields = ("name", "email", "body")
    date_hierarchy = "time_created"
    ordering = ("-time_created",)
    fieldsets = (
        (None, {"fields": (("name", "email", "website"), "body", 
            ("ip_address", "approved"), ("blog_post", "replied_to"))}),
    )

admin.site.register(BlogPost, BlogPostAdmin)
if not COMMENTS_DISQUS_SHORTNAME:
    admin.site.register(Comment, CommentAdmin)




from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin


admin.autodiscover()

urlpatterns = patterns("",
    ("^admin/", include(admin.site.urls)),
    ("^", include("mezzanine.urls")),
)

if getattr(settings, "DEV_SERVER", False):
    urlpatterns += patterns("",
        ("^%s/(?P<path>.*)$" % settings.MEDIA_URL.strip("/"), 
            "django.views.static.serve", {"document_root": settings.MEDIA_ROOT}),
        ("^favicon.ico$", "django.views.static.serve", {"document_root": 
            settings.MEDIA_ROOT, "path": "img/favicon.ico"}),
    )





from datetime import datetime

from django.core.urlresolvers import reverse
from django.db import models
from django.db.models.base import ModelBase
from django.template.defaultfilters import slugify, truncatewords_html
from django.utils.translation import ugettext, ugettext_lazy as _

from mezzanine.core.managers import PublishedManager
from mezzanine import settings as blog_settings


class Displayable(models.Model):
    """
    Abstract model representing a visible object on the website with common 
    functionality such as auto slug creation and an active field for toggling 
    visibility.
    """

    title = models.CharField(_("Title"), max_length=100)
    content = models.TextField(_("Content"))
    description = models.TextField(_("Description"), blank=True)
    status = models.IntegerField(_("Status"), 
        choices=blog_settings.CONTENT_STATUS_CHOICES, 
        default=blog_settings.CONTENT_STATUS_DRAFT)
    publish_date = models.DateTimeField(_("Published from"), blank=True, 
        help_text=_("With published selected, won't be shown until this time"), 
        default=datetime.now)
    slug = models.SlugField(max_length=100, editable=False, null=True)
    keywords = models.ManyToManyField("Keyword", verbose_name=_("Keywords"),
        blank=True)
    short_url = models.URLField(blank=True, null=True)
        
    objects = PublishedManager()

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.title
        
    def save(self, *args, **kwargs):
        """
        Create the description from the content if none given, and create a 
        unique slug from the title by appending an index.
        """
        if self.publish_date is None:
            # publish_date will be blank when a blog post is created from the 
            # quick blog form in the admin dashboard.
            self.publish_date = datetime.now()
        if not self.description:
            for s in ("</p>", "\n", ". "):
                if s in self.content:
                    self.description = self.content.split(s)[0] + s
                    break
            else:
                self.description = truncatewords_html(self.content, 100)
        if self.id is None:
            self.slug = self.get_slug()
            i = 0
            while True:
                if i > 0:
                    self.slug = "%s-%s" % (self.slug, i)
                if not self.__class__.objects.filter(slug=self.slug):
                    break
                i += 1
        super(Displayable, self).save(*args, **kwargs)
        
    def get_slug(self):
        return slugify(self.title)
    
    def admin_link(self):
        return "<a href='%s'>%s</a>" % (self.get_absolute_url(), 
            ugettext("View on site"))
    admin_link.allow_tags = True
    admin_link.short_description = ""

class OrderableBase(ModelBase):
    """
    Checks for order_with_respect_to on the model's inner Meta class and if 
    found, copies it to a custom attribute and deletes it since it will cause 
    errors when used with ForeignKey("self"). Also specify ordering on the Meta 
    class if not yet provided.
    """

    def __new__(cls, name, bases, attrs):
        if "Meta" not in attrs:
            attrs["Meta"] = object()
        if hasattr(attrs["Meta"], "order_with_respect_to"):
            attrs["order_with_respect_to"] = attrs["Meta"].order_with_respect_to
            del attrs["Meta"].order_with_respect_to
        if not hasattr(attrs["Meta"], "ordering"):
            setattr(attrs["Meta"], "ordering", ("_order",))
        return super(OrderableBase, cls).__new__(cls, name, bases, attrs)

class Orderable(models.Model):
    """
    Provide a custom ordering integer field similar to using Meta's 
    order_with_respect_to since to date (Django 1.2) order_with_respect_to 
    doesn't work with ForeignKey("self"). We may also want this feature for 
    models that aren't ordered with respect to a particular field.
    """

    __metaclass__ = OrderableBase
    
    _order = models.IntegerField(_("Order"), null=True)

    class Meta:
        abstract = True
        
    def with_respect_to(self):
        """
        Returns a dict to use as a filter for ordering operations containing 
        the original Meta.order_with_respect_to value if provided.
        """
        try:
            field = self.order_with_respect_to
            return {field: getattr(self, field)}
        except AttributeError:
            return {}
        
    def save(self, *args, **kwargs):
        """
        Set the initial ordering value.
        """
        if self._order is None:
            lookup = self.with_respect_to()
            self._order = self.__class__.objects.filter(**lookup).count()
        super(Orderable, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        """
        Update the ordering values for siblings.
        """
        lookup = self.with_respect_to()
        lookup["_order__gte"] = self._order
        after = self.__class__.objects.filter(**lookup)
        after.update(_order=models.F("_order") - 1)
        super(Orderable, self).delete(*args, **kwargs)

class Keyword(models.Model):
    """
    Keywords/tags which are managed via a custom Javascript based widget in the 
    admin.
    """

    class Meta:
        verbose_name = "Keyword"
        verbose_name_plural = "Keywords"
        ordering = ("value",)

    value = models.CharField(max_length=50)

    def __unicode__(self):
        return self.value





from urllib import urlopen, urlencode

from django import template
from django.conf import settings
from django.utils.simplejson import loads

from mezzanine import settings as mezzanine_settings


register = template.Library()


@register.simple_tag
def setting(setting_name):
    """
    Return a setting.
    """
    value = getattr(mezzanine_settings, setting_name, 
        getattr(settings, setting_name, None))
    if value is None:
        value = ""
    return value

def register_as_tag(register):
    """
    Decorator that creates a tag with the format: {% func_name as var_name %}
    The decorated func returns the value that is given to var_name in the 
    template context.
    """
    def wrapper(func):
        def tag(parser, token):
            class TagNode(template.Node):
                def render(self, context):
                    parts = token.split_contents()
                    context[parts[-1]] = func(*parts[1:-2])
                    return ""
            return TagNode()
        for copy_attr in ("__dict__", "__doc__", "__name__"):
            setattr(tag, copy_attr, getattr(func, copy_attr))
        return register.tag(tag)
    return wrapper

def register_render_tag(register):
    """
    Decorator that creates a template tag using the given renderer as the 
    render function for the template tag node - the render function takes two 
    arguments - the template context and the tag token.
    """
    def wrapper(renderer):
        def tag(parser, token):
            class TagNode(template.Node):
                def render(self, context):
                    return renderer(context, token)
            return TagNode()
        for copy_attr in ("__dict__", "__doc__", "__name__"):
            setattr(tag, copy_attr, getattr(renderer, copy_attr))
        return register.tag(tag)
    return wrapper

@register_render_tag(register)
def set_short_url_for(context, token):
    """
    Sets the ``short_url`` attribute of the given model using the bit.ly 
    credentials if they have been specified and saves it.
    """
    obj = context[token.split_contents()[1]]
    request = context["request"]
    if getattr(obj, "short_url") is None:
        obj.short_url = request.build_absolute_uri(request.path)
        args = {
            "login": getattr(mezzanine_settings, "BLOG_BITLY_USER"),
            "apiKey": getattr(mezzanine_settings, "BLOG_BITLY_KEY"),
            "longUrl": obj.short_url,
        }
        if args["login"] and args["apiKey"]:
            url = "http://api.bit.ly/v3/shorten?%s" % urlencode(args)
            response = loads(urlopen(url).read())
            if response["status_code"] == 200:
                obj.short_url = response["data"]["url"]
                obj.save()
    return ""

@register_render_tag(register)
def admin_reorder(context, token):
    """
    Called in admin/base_site.html template override and applies custom ordering 
    of apps/models defined by settings.ADMIN_REORDER.
    """
    # sort key function - use index of item in order if exists, otherwise item
    sort = lambda order, item: (order.index(item), "") if item in order else (
        len(order), item) 
    if "app_list" in context:
        # sort the app list
        order = SortedDict(mezzanine_settings.ADMIN_REORDER)
        context["app_list"].sort(key=lambda app: sort(order.keys(), 
            app["app_url"][:-1]))
        for i, app in enumerate(context["app_list"]):
            # sort the model list for each app
            app_name = app["app_url"][:-1]
            if not app_name:
                app_name = context["request"].path.strip("/").split("/")[-1]
            model_order = [m.lower() for m in order.get(app_name, [])]
            context["app_list"][i]["models"].sort(key=lambda model: 
                sort(model_order, model["admin_url"].strip("/").split("/")[-1]))
    return ""





import os.path

from django.conf import settings
from django.utils.translation import ugettext_lazy as _


# Unregister these models installed by default (occurs in urlconf).
ADMIN_REMOVAL = ()

# Blog feed settings.
BLOG_TITLE = "The Mezzanine Blog"
BLOG_DESCRIPTION = "The Mezzanine Blog"

# Credentials for bit.ly URL shortening service.
BLOG_BITLY_USER = None
BLOG_BITLY_KEY = None

# Number of blog posts to show on a blog listing page.
BLOG_POST_PER_PAGE = 5

# Maximum number of paging links to show on a blog listing page.
BLOG_POST_MAX_PAGING_LINKS = 10

# Slug of the page object for the blog.
BLOG_SLUG = "blog"

# Shortname when using the Disqus comments system (http://disqus.com).
COMMENTS_DISQUS_SHORTNAME = None

# Disqus user's API key for displaying recent comments in the admin dashboard.
COMMENTS_DISQUS_KEY = None

# If True, the built-in comments are approved by default.
COMMENTS_DEFAULT_APPROVED = True

# Number of latest comments to show in the admin dashboard.
COMMENTS_NUM_LATEST = 5

# If True, unapproved comments will have a placeholder visible on the site 
# with a "waiting for approval" or "comment removed" message based on the 
# workflow around the COMMENTS_DEFAULT_APPROVED setting - if True then the 
# former message is used, if False then the latter.
COMMENTS_UNAPPROVED_VISIBLE = True

# Media files for admin.
CONTENT_MEDIA_PATH = os.path.join(os.path.dirname(__file__), "core", "media")
CONTENT_MEDIA_URL = "/content_media/"

# Content status choices.
CONTENT_STATUS_DRAFT = 1
CONTENT_STATUS_PUBLISHED = 2
CONTENT_STATUS_CHOICES = (
    (CONTENT_STATUS_DRAFT, _("Draft")),
    (CONTENT_STATUS_PUBLISHED, _("Published")),
)

# ID for using Google Analytics (http://www.google.com/analytics/) referred to 
# as "Web Property ID"
GOOGLE_ANALYTICS_ID = None

# Strings to check for in user agents when testing for a mobile device.
MOBILE_USER_AGENTS = ("2.0 MMP","240x320","400X240","AvantGo","BlackBerry",
    "Blazer","Cellphone","Danger","DoCoMo","Elaine/3.0","EudoraWeb",
    "Googlebot-Mobile","hiptop","IEMobile","KYOCERA/WX310K","LG/U990",
    "MIDP-2.","MMEF20","MOT-V","NetFront","Newt","Nintendo Wii","Nitro",
    "Nokia","Opera Mini","Palm","PlayStation Portable","portalmmm","Proxinet",
    "ProxiNet","SHARP-TQ-GX10","SHG-i900","Small","SonyEricsson","Symbian OS",
    "SymbianOS","TS21i-10","UP.Browser","UP.Link","webOS","Windows CE",
    "WinWAP","YahooSeeker/M1A1-R2D2","iPhone","iPod","Android",
    "BlackBerry9530","LG-TU915 Obigo","LGE VX","webOS","Nokia5800")

# Number of different sizes given to tags when shown as a cloud.
TAG_CLOUD_SIZES = 4

# Store these package names here as they may change in the future since at the 
# moment we are using custom forks of them.
PACKAGE_NAME_FILEBROWSER = "filebrowser_safe"
PACKAGE_NAME_GRAPPELLI = "grappelli_safe"

# If True, the pages menu will show all levels of navigation by default, 
# otherwise child pages are only shown when viewing the parent page.
PAGES_MENU_SHOW_ALL = True





from collections import defaultdict

from django import template
from django.core.urlresolvers import reverse
from django.db.models import get_model, get_models

from mezzanine.core.templatetags.mezzanine_tags import register_as_tag
from mezzanine.pages.models import Page
from mezzanine.settings import PAGES_MENU_SHOW_ALL


register = template.Library()


def _page_menu(context, parent_page):
    """
    Return a list of child pages for the given parent, storing all 
    pages in a dict in the context when first called using parents as keys 
    for retrieval on subsequent recursive calls from the menu template.
    """
    if "menu_pages" not in context:
        pages = defaultdict(list)
        try:
            user = context["request"].user
        except KeyError:
            user = None
        for page in Page.objects.published(for_user=user).order_by("_order"):
            try:
                slug = context["request"].path.strip("/")
            except KeyError:
                slug = ""
            setattr(page, "selected", slug.startswith(page.slug))
            pages[page.parent_id].append(page)
        context["menu_pages"] = pages
    if parent_page is not None:
        parent_page = parent_page.id
    context["page_branch"] = context["menu_pages"].get(parent_page, [])
    context["PAGES_MENU_SHOW_ALL"] = PAGES_MENU_SHOW_ALL
    return context

@register.inclusion_tag("pages/includes/page_menu.html", takes_context=True)
def page_menu(context, parent_page=None):
    """
    Public page menu.
    """
    return _page_menu(context, parent_page)

@register.inclusion_tag("admin/includes/page_menu.html", takes_context=True)
def page_menu_admin(context, parent_page=None):
    """
    Admin page menu.
    """
    return _page_menu(context, parent_page)

@register_as_tag(register)
def models_for_pages(*args):
    """
    Create a select list containing each of the models that subclass Page
    """
    page_models = []
    for model in get_models():
        if issubclass(model, Page):
            setattr(model, "name", model._meta.verbose_name)
            setattr(model, "add_url", reverse("admin:%s_%s_add" % 
                (model._meta.app_label, model.__name__.lower())))
            page_models.append(model)
    return page_models

@register.filter
def is_page_content_model(admin_model_dict):
    """
    Returns True if the model in the given admin dict is a subclass of Page.
    """
    args = admin_model_dict["admin_url"].strip("/").split("/")
    if len(args) == 2:
        model = get_model(*args)
        return model is not Page and issubclass(model, Page)
    return False




#!/usr/bin/env python

import os
import sys
import shutil
import mezzanine

def create_project():

    script_name = os.path.basename(sys.argv[0])
    if len(sys.argv) == 1:
        print
        print "Usage: %s project_name" % script_name
        print
        sys.exit()
        
    mezzanine_path = os.path.dirname(os.path.abspath(mezzanine.__file__))
    from_path = os.path.join(mezzanine_path, "project_template")
    to_path = os.path.join(os.getcwd(), sys.argv[1])

    try:
        shutil.copytree(from_path, to_path)
    except Exception, e:
        print
        print "Error in %s: %s" % (script_name, e)
        print
        
if __name__ == "__main__":
    create_project()




from django.conf.urls.defaults import *

from mezzanine.blog.feeds import PostsRSS, PostsAtom


# Blog feed patterns.
blog_feed_dict = {"rss": PostsRSS, "atom": PostsAtom}
urlpatterns = patterns("",
    url("^feeds/(?P<url>.*)/$", "django.contrib.syndication.views.feed", 
        {"feed_dict": blog_feed_dict}, name="blog_post_feed"),
)

# Blog patterns.
urlpatterns += patterns("mezzanine.blog.views",
    url("^tag/(?P<tag>.*)/$", "blog_post_list", name="blog_post_list_tag"),
    url("^archive/(?P<year>\d{4})/(?P<month>\d{1,2})/$", "blog_post_list", 
        name="blog_post_list_month"),
    url("^author/(?P<username>.*)/$", "blog_post_list", 
        name="blog_post_list_author"),
    url("^archive/(?P<year>.*)/$", "blog_post_list", 
        name="blog_post_list_year"),
    url("^(?P<slug>.*)/$", "blog_post_detail", name="blog_post_detail"),
    url("^$", "blog_post_list", name="blog_post_list"),
)





from django.shortcuts import get_object_or_404

from mezzanine.forms.models import BuiltForm, BuiltFormSubmission, STATUS_PUBLIC


def form_builder_optional_fields():
	"""
	return the field name / verbose name pairs for all fields in 
	BuiltFormSubmission that are considered optional, eg blank=True
	"""

	meta = BuiltFormSubmission._meta
	return [(field.name, field.verbose_name) for field in meta.fields 
		if field.blank and field != meta.pk]	


def get_built_form(request, slug):
	"""
	wrap get_object_or_404 with a check against status if the user is not staff
	"""
	
	queryset = BuiltForm.objects.all()
	if not request.user.is_staff:
		queryset = queryset.filter(status=STATUS_PUBLIC)
	return get_object_or_404(queryset, slug=slug)




from setuptools import setup, find_packages
import os

exclude = ["mezzanine/project_template/mezzanine.db",]
exclude = dict([(e, None) for e in exclude])
for e in exclude:
    if e.endswith(".py"):
        try:
            os.remove("%sc" % e)
        except:
            pass
    try:
        with open(e, "r") as f:
            exclude[e] = (f.read(), os.stat(e))
        os.remove(e)
    except:
        pass

try:
    setup(

        name = "Mezzanine",
        version = __import__("mezzanine").__version__,
        author = "Stephen McDonald",
        author_email = "stephen.mc@gmail.com",
        description = "A content management platform built using the Django framework.",
        long_description = open("README.rst").read(),
        license = "BSD",
        url = "http://mezzanine.jupo.org/",
        zip_safe = False,
        include_package_data = True,
        packages = find_packages(),

        install_requires = [
            "setuptools",
            "grappelli_safe",
            "filebrowser_safe",
            "PIL",
        ],

        entry_points = """
            [console_scripts]
            mezzanine-project=mezzanine.bin.mezzanine_project:create_project
        """,

        classifiers = [
            "Development Status :: 4 - Beta",
            "Environment :: Web Environment",
            "Framework :: Django",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Topic :: Internet :: WWW/HTTP",
            "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
            "Topic :: Internet :: WWW/HTTP :: WSGI",
            "Topic :: Software Development :: Libraries :: Application Frameworks",
            "Topic :: Software Development :: Libraries :: Python Modules",
        ]

    )

finally:
    for e in exclude:
        if exclude[e] is not None:
            data, stat = exclude[e]
            try:
                with open(e, "w") as f:
                    f.write(data)
                os.chown(e, stat.st_uid, stat.st_gid)
                os.chmod(e, stat.st_mode)
            except:
                pass





from datetime import date

from django import forms
from django.forms.extras import SelectDateWidget

from mezzanine.forms.models import BuiltForm, BuiltFormSubmission
from mezzanine.forms.utils import form_builder_optional_fields


class DobWidget(SelectDateWidget):
	"""
	populate the years of a SelectDateWidget for selecting dob
	"""
	
	def __init__(self, attrs=None):
		year = date.today().year
		self.attrs = attrs
		self.years = range(year, year - 100, -1)
		super(SelectDateWidget, self).__init__()


def get_built_form_form(built_form):
	"""
	return a model form for a BuiltFormSubmission, excluding the extra
	fields that haven't been selected as optional or mandatory for the given 
	BuiltForm, and setting the extra mandatory fields that have been selected 
	as mandatory fields
	"""
	
	dob_field = None
	extra_fields = (built_form.extra_fields("mandatory") + 
		built_form.extra_fields("optional"))
	if "dob" in extra_fields:
		dob_field = forms.DateField(widget=DobWidget)

	class BuiltFormForm(forms.ModelForm):
	
		class Meta:
			model = BuiltFormSubmission
			exclude = ["submission_date","built_form"] + [field[0] for field in 
				form_builder_optional_fields() if field[0] not in extra_fields]
			
		dob = dob_field
		
		def __init__(self, *args, **kwargs):
			super(BuiltFormForm, self).__init__(*args, **kwargs)
			for field in built_form.extra_fields("mandatory"):
				self.fields[field].required = True
				if hasattr(self.fields[field], "choices"):
					self.fields[field].choices = self.fields[field].choices[1:]

	return BuiltFormForm





from django.conf.urls.defaults import patterns, url
from mezzanine.forms.views import built_form_detail, built_form_sent

urlpatterns = patterns("",
    url(r'(?P<slug>.*)/sent/$', built_form_sent, name="form_sent"),
    url(r'(?P<slug>.*)/$', built_form_detail, name="form_detail"),
)






from django.contrib.syndication.feeds import Feed
from django.core.urlresolvers import reverse
from django.utils.feedgenerator import Atom1Feed

from mezzanine.blog.models import BlogPost
from mezzanine.settings import BLOG_TITLE, BLOG_DESCRIPTION


class PostsRSS(Feed):
    """
    RSS feed for all blog posts.
    """

    title = BLOG_TITLE
    description = BLOG_DESCRIPTION
    
    def link(self):
        return reverse("blog_post_feed", kwargs={"url": "rss"})
    
    def items(self):
        return BlogPost.objects.published()

class PostsAtom(PostsRSS):
    """
    Atom feed for all blog posts.
    """
    
    feed_type = Atom1Feed
    subtitle = PostsRSS.description

    def link(self):
        return reverse("blog_post_feed", kwargs={"url": "atom"})





from django.conf import settings
from django.conf.urls.defaults import *

from mezzanine.settings import BLOG_SLUG


urlpatterns = patterns("", 
    ("^", include("mezzanine.core.urls")),
)

if "mezzanine.blog" in settings.INSTALLED_APPS:
    urlpatterns += patterns("", 
        ("^%s/" % BLOG_SLUG, include("mezzanine.blog.urls")),
    )
if "mezzanine.forms" in settings.INSTALLED_APPS:
    urlpatterns += patterns("", 
        ("^", include("mezzanine.forms.urls")),
    )
if "mezzanine.pages" in settings.INSTALLED_APPS:
    urlpatterns += patterns("", 
        ("^", include("mezzanine.pages.urls")),
    )





from django.core.urlresolvers import resolve
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

from mezzanine.core.models import Displayable, Orderable


class Page(Orderable, Displayable):
    """
    A page in the page tree.
    """
    
    parent = models.ForeignKey("Page", blank=True, null=True, 
        related_name="children")
    titles = models.CharField(editable=False, max_length=1000, null=True)
    content_model = models.CharField(editable=False, max_length=50, null=True)

    class Meta:
        verbose_name = _("Page")
        verbose_name_plural = _("Pages")
        ordering = ("titles",)
        order_with_respect_to = "parent"

    def __unicode__(self):
        return self.titles
        
    @models.permalink
    def get_absolute_url(self):
        return ("page", (), {"slug": self.get_slug()})

    def save(self, *args, **kwargs):
        """
        Create the titles field using the titles up the parent chain and set 
        the initial value for ordering.
        """
        if self.id is None:
            titles = [self.title]
            parent = self.parent
            while parent is not None:
                titles.insert(0, parent.title)
                parent = parent.parent
            self.titles = " / ".join(titles)
            self.content_model = self.__class__.__name__.lower()
        super(Page, self).save(*args, **kwargs)
    
    def get_content_model(self):
        return getattr(self, self.content_model, None)

    def get_slug(self):
        """
        Recursively build the slug from the chain of parents.
        """
        slug = slugify(self.title)
        if self.parent is not None:
            return "%s/%s" % (self.parent.get_slug(), slug)
        return slug
        
    def overridden(self):
        """
        Return True if the page's slug has an explicitly defined url pattern 
        and is therefore considered to be overriden.
        """
        from mezzanine.pages.views import page
        resolved_view = resolve(self.get_absolute_url())[0]
        return resolved_view != page





from copy import deepcopy

from django.contrib import admin
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect

from mezzanine.core.admin import OrderableAdmin
from mezzanine.forms.models import Form, Field
from mezzanine.pages.admin import PageAdmin


form_fieldsets = deepcopy(PageAdmin.fieldsets)
form_fieldsets[0][1]["fields"] += ("response", "send_email")
FormMedia = deepcopy(OrderableAdmin.Media)
FormMedia.js = PageAdmin.Media.js + [js for js in FormMedia.js if js not in 
    PageAdmin.Media.js]

class FieldAdmin(admin.TabularInline):
    model = Field
   
class FormAdmin(PageAdmin, OrderableAdmin):

    class Media(FormMedia):
        pass

    inlines = (FieldAdmin,)
    fieldsets = form_fieldsets
    
admin.site.register(Form, FormAdmin)
